/*
 *
 * Driver for the MMCR registers on the AMD Elan sc520
 *
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <sys/types.h>
#include <sys/conf.h>


#include <uvm/uvm_extern.h>

#include <arch/i386/mmcr/mmcrreg.h>
#include <arch/i386/mmcr/mmcrvar.h>

#include "opt_cputype.h"
#include "skled.h"
#include "wdog_mmcr.h"

#if NSKLED > 0
#include <arch/i386/mmcr/skledvar.h>
#endif

#if NWDOG_MMCR > 0
#include <arch/i386/mmcr/wdog_mmcrvar.h>
#endif

union mmcrdev_attach_args {
	const char *maa_busname;
#if NSKLED > 0
	struct skled_attach_args maa_skaa;
#endif
#if NWDOG_MMCR > 0
	struct wdog_mmcr_attach_args maa_wdaa;
#endif
};


#if !defined(I486_CPU)
#error "The mmcr driver only works with AMD Elan 486's"
#endif

static int
mmcrmatch(struct device *parent, struct cfdata *match, void *aux);

static void
mmcrattach(struct device *parent, struct device *self, void *aux);

int mmcr_print(void *aux, const char *pnp);

struct mmcr_softc {
	struct device 	sc_dev;
	/* the MMCR base address after we've mapped it in */
	caddr_t		mmcr_base;
};

struct cfattach mmcr_ca = {
	sizeof(struct mmcr_softc), mmcrmatch, mmcrattach
};

extern struct cfdriver mmcr_cd;
extern caddr_t mmcr_base;
extern int cpu_id;

static int
mmcrmatch(struct device *parent, struct cfdata *match, void *aux)
{
	struct mmcr_attach_args *maa = aux;

	if (strcmp(maa->maa_busname, "mmcr") != 0)
		return(0);

	/* XXX use real values, not a magic number */
	/* the cpu_id changes depending on weather it's a warm or cold boot */
	if (cpu_class == CPUCLASS_486 && (cpu_id == 0x4f4 || cpu_id == 0x494))
		return(1);

	printf("mmcr : cpu_id 0x%04x\n", cpu_id);
	printf("\nMMCR registers not present on this CPU\n");
	return(0);
}

static void
mmcrattach(struct device *parent, struct device *self, void *aux)
{
	struct mmcr_softc *sc = (void *)self;
	union mmcrdev_attach_args	maa;

	maa.maa_busname = "mmcr";

	/* XXX find out about uvm_map_reserve (?) */
	pmap_enter(pmap_kernel(), (vaddr_t)(mmcr_base),
		(paddr_t)MMCR_BASE_ADDR,
		VM_PROT_READ|VM_PROT_WRITE,
		VM_PROT_READ|VM_PROT_WRITE|PMAP_WIRED);

	/* tell the pmap to sync up now */
	pmap_update(pmap_kernel());

	sc->mmcr_base = mmcr_base;

	printf("\n");

#if NSKLED > 0
	maa.maa_skaa.skaa_busname = "skled";
	config_found(self, &maa.maa_skaa, mmcr_print);
#endif

#if NWDOG_MMCR > 0
	maa.maa_wdaa.wdaa_busname = "wdog_mmcr";
	config_found(self, &maa.maa_wdaa, mmcr_print);
#endif

}

int
mmcr_print(void *aux, const char *pnp)
{
	union mmcrdev_attach_args *maa = aux;

	if (pnp)
		printf("%s at %s", maa->maa_busname, pnp);
	return (UNCONF);
}
