/*
 *
 * 'Driver' for the 'Error' LED on the Soekris Net45[20]1
 *
 */

#include <sys/param.h>
#include <sys/device.h>
#include <sys/types.h>
#include <sys/conf.h>
#include <sys/systm.h>

#include <arch/i386/mmcr/skledvar.h>
#include <arch/i386/mmcr/mmcrreg.h>

#define LED_BIT		0x0200

static int
skledmatch(struct device *parent, struct cfdata *match, void *aux);

static void
skledattach(struct device *parent, struct device *self, void *aux);

cdev_decl(skled);

struct skled_softc {
	struct device 	sc_dev;
	/* the MMCR base address after we've mapped it in */
	caddr_t		mmcr_base;
	u_char		attached;
};

struct cfattach skled_ca = {
	sizeof(struct skled_softc), skledmatch, skledattach
};

extern struct cfdriver skled_cd;

extern caddr_t mmcr_base;

#define SKLED_GET \
	(*(u_int32_t *)(sc->mmcr_base + MMCR_GPIO_DATA) & LED_BIT)
#define SKLED_SET \
	(*(u_int32_t *)(sc->mmcr_base + MMCR_GPIO_SET) = LED_BIT)
#define SKLED_CLEAR \
	(*(u_int32_t *)(sc->mmcr_base + MMCR_GPIO_CLEAR) = LED_BIT)


static int
skledmatch(struct device *parent, struct cfdata *match, void *aux)
{
	struct skled_attach_args *skaa = aux;

	if (strcmp(skaa->skaa_busname, "skled") != 0)
		return (0);

	/* check that LED_BIT is in GPIO mode and an output. */
	if ( !(*(u_int32_t *)(mmcr_base + MMCR_GPIO_IORO) & LED_BIT) &&
	      (*(u_int32_t *)(mmcr_base + MMCR_GPIO_FUNCSEL) & LED_BIT)
	   ){
	   	printf("skled: led not in GPIO mode or output mode\n");
	   	return (0);
	   }

	return (1);
}

static void
skledattach(struct device *parent, struct device *self, void *aux)
{
	struct skled_softc *sc = (void *)self;

	/* should already be set for us */
	sc->mmcr_base = mmcr_base;

	printf(": LED %s\n", SKLED_GET ? "On" : "Off");
	
	sc->attached = 1;
	
	/* If the led is already on, switch if off */
	SKLED_CLEAR;
}

int
skledopen(dev_t dev, int flags, int fmt, struct proc *p)
{
	struct skled_softc *sc;
	int unit = minor(dev);

	if (unit != 0)
		return (ENXIO);
	
	sc = skled_cd.cd_devs[unit];

	if (!sc)
		return (ENXIO);

	if (!sc->attached)
		return (ENXIO);

	return (0);
}

int
skledclose(dev_t dev, int flags, int fmt, struct proc *p)
{
	return(0);
}

int
skledread(dev_t dev, struct uio *uio, int flag)
{
	struct skled_softc *sc;
	int unit = minor(dev);
	char buf[2];
	int xlen;

	if (unit != 0)
		return (ENXIO);

	sc = skled_cd.cd_devs[unit];

	if (!sc)
		return (ENXIO);

	if (!sc->attached)
		return (ENXIO);
	
	if (uio->uio_offset >= sizeof buf)
		return (0);
	
	buf[0] = SKLED_GET ? '1' : '0';
	buf[1] = '\n';
	
	xlen = sizeof(buf) - uio->uio_offset;
	if (xlen > uio->uio_resid)
		xlen = uio->uio_resid;
		
	return uiomove((caddr_t)buf, xlen, uio);
}

int
skledwrite(dev_t dev, struct uio *uio, int flag)
{
	struct skled_softc *sc;
	int unit = minor(dev);
	char buf[2];
	int cnt, error;

	if (unit != 0)
		return (ENXIO);
	
	sc = skled_cd.cd_devs[unit];

	if (!sc)
		return (ENXIO);

	if (!sc->attached)
		return (ENXIO);

	cnt = uio->uio_resid;
	if (uio->uio_offset || (cnt != sizeof buf && cnt != sizeof buf - 1))
		return (EINVAL);
	
	if ((error = uiomove((caddr_t)buf, sizeof buf, uio)) != 0)
		return (EINVAL);
	
	switch (buf[0]){
		case 't': /* FALLTHROUGH */
		case 'T': /* toggle current state*/
			if (SKLED_GET) SKLED_CLEAR; else SKLED_SET;
			break;
		case '1':
			SKLED_SET;
			break;
		case '0':
			SKLED_CLEAR;
			break;
		default:
			return (EINVAL);
	}

	return(0);
}

